<?php

namespace Drupal\classes_extractor\Plugin\ClassesExtractor;

use Drupal\classes_extractor\ClassesExtractorBase;
use Drupal\classes_extractor\ClassesExtractorManager;
use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\layout_builder\SectionComponent;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a plugin to extract classes based on entity display.
 *
 * @ClassesExtractor(
 *   id = "entity_display_classes_extractor",
 *   label = @Translation("Entity Display Classes Extractor"),
 * )
 */
class EntityDisplayClassesExtractor extends ClassesExtractorBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;


  /**
   * The entity type manager service.
   *
   * @var \Drupal\classes_extractor\ClassesExtractorManager
   */
  protected $classExtractorManager;

  /**
   * Constructs an EntityDisplayClassesExtractor object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $class_extractor_manager
   *   The entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ClassesExtractorManager $class_extractor_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->classExtractorManager = $class_extractor_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('classes_extractor.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getClasses(): array {
    $classes = [];
    $displays = $this->entityTypeManager->getStorage('entity_view_display')->loadMultiple();
    foreach ($displays as $display) {
      if ($display instanceof EntityViewDisplay) {
        $components = [
          $display->getComponents(),
          $display->getThirdPartySetting('ds', 'fields', []),
          $display->getThirdPartySetting('ds', 'layout', []),
          $display->getThirdPartySetting('ds', 'regions', []),
          $display->getThirdPartySettings('field_group'),
        ];
        // Handle Layout Builder components if module exist.
        if (\Drupal::moduleHandler()->moduleExists('layout_builder')) {
          $components[] = $display->getThirdPartySetting('layout_builder', 'sections', []);
          $layoutBuilderSections = $display->getThirdPartySetting('layout_builder', 'sections', []);
        }
        foreach ($components as $component) {
          $this->classExtractorManager->recursiveGetClasses($component, $classes);
        }
        foreach ($layoutBuilderSections as $section) {
          foreach ($section->getComponents() as $component) {
            if ($component instanceof SectionComponent) {
              $this->classExtractorManager->recursiveGetClasses($component->get("configuration"), $classes);
            }
          }
        }
      }
    }
    return $classes;
  }

}
